using System;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.Xml.XPath;

namespace UserStageExample.FlexiService
{
    /// <summary>
    /// Represents an XML elements of a Document document
    /// </summary>
    public class Document
    {
        private const string Prefix = "d";
        private const string Namespace = "urn:docData";

        private const string SectionsQuery = "/d:Document/d:Sections";

        private const string PagesQuery = "d:Document/d:Pages/d:Page";

        private const string AddressingQuery =
            "d:Field[@Name='Page 1']/d:Children/d:Field[@Name='Addressing']/d:ChoiceValue";

        private const string FirstNameQuery =
                "d:Field[@Name='Page 1']/d:Children/d:Field[@Name='Name']/d:Children/d:Field[@Name='First Name']/d:TextValue";

        private const string LastNameQuery =
            "d:Field[@Name='Page 1']/d:Children/d:Field[@Name='Name']/d:Children/d:Field[@Name='Last Name']/d:TextValue";

        private const string ApprovalDateQuery =
            "d:Field[@Name='Page 2']/d:Children/d:Field[@Name='Approval Date']/d:TextValue";


        private const string ChoiceValue = "Value";
        private const string TextValue = "Text";

        /// <summary>
        /// An XML document of the task
        /// </summary>
        private readonly XDocument _xml;

        private readonly XmlNamespaceManager _ns;
        private readonly XElement _sections;

        /// <summary>
        /// A document ID
        /// </summary>
        public int Id { get; private set; }

        public void WriteXml(XmlWriter writer)
        {
            _xml.WriteTo(writer);
        }

        public string GetXml()
        {
            var builder = new StringBuilder();
            using (var writer = XmlWriter.Create(builder, new XmlWriterSettings() { Encoding = Encoding.Unicode }))
            {
                WriteXml(writer);
            }
            return builder.ToString();
        }

        public Document(int id, XDocument xml)
        {
            _xml = xml;
            Id = id;
            _ns = new XmlNamespaceManager(new NameTable());
            _ns.AddNamespace(Prefix, Namespace);
            _sections = _xml.XPathSelectElement(SectionsQuery, _ns);
            if (_xml.Root == null || _sections == null) throw new Exception("Invalid document");
        }

        /// <summary>
        /// Forms the full name of an applicant specified on the first list
        /// </summary>
        /// <returns>The full name of the applicant with the title</returns>
        public string PersonName()
        {
            var addressing = (string)_sections.XPathSelectElement(AddressingQuery, _ns).Attribute(ChoiceValue);
            var firstName = (string)_sections.XPathSelectElement(FirstNameQuery, _ns).Attribute(TextValue);
            var secondName = (string)_sections.XPathSelectElement(LastNameQuery, _ns).Attribute(TextValue);

            return string.Format("{0} {1} {2}", addressing, firstName, secondName);
        }

        /// <summary>
        /// Sets the Approval Date field to display the current date
        /// </summary>
        public void CreateStamp()
        {
            _xml.Root.SetAttributeValue("Unchanged", false);
            var approvalDate = _sections.XPathSelectElement(ApprovalDateQuery, _ns);
            approvalDate.SetAttributeValue(TextValue, DateTime.Now.ToString("MM/dd/yyyy"));
        }
    }
}