﻿using System;
using System.Windows;
using System.Windows.Data;

namespace UserStageExample.ValueConverters
{
    /// <summary>
    /// A converter of boolean values to the element visibility
    /// </summary>
    internal class BooleanToVisibilityConverter : IValueConverter
    {
        /// <summary>
        /// Converts a boolean value to a visibility type
        /// </summary>
        /// <param name="value">Original value</param>
        /// <param name="targetType">Target type</param>
        /// <param name="parameter">Not used</param>
        /// <param name="culture">Not used</param>
        /// <returns></returns>
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value is bool && (bool)value)
            {
                return Visibility.Visible;
            }
            return Visibility.Collapsed;
        }

        /// <summary>
        /// Converts a visiblility type to a boolean value
        /// </summary>
        /// <param name="value">Visibility type</param>
        /// <param name="targetType">Target type</param>
        /// <param name="parameter">Not used</param>
        /// <param name="culture">Not used</param>
        /// <returns></returns>
        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            return value is Visibility && (Visibility)value == Visibility.Visible;
        }
    }
}
